@@ -11,6 +11,8 @@ import anndata
 import attr
 import scanpy as sc
 import scipy.sparse
+import re
+import scipy.io
 import pandas as pd
 from logzero import logger
 
@@ -222,6 +224,9 @@ class CellRangerConverter:
         return ad
 
     def _write_output(self, ad):
+        if self.args.ncells:
+            logger.info("Sampling %s cells from anndata object", self.args.ncells)
+            sc.pp.subsample(ad, n_obs=self.args.ncells)
         logger.info("Saving anndata object to %s", self.args.out_file)
         ad.write(self.args.out_file)
 
@@ -290,23 +295,45 @@ class TextConverter:
             return pd.read_csv(marker_file, header=0, sep="\t")
 
     def _load_expression(self, coords, annotation, markers):
-        expression_file = os.path.join(self.args.indir, "expression.tsv.gz")
-        if not os.path.isfile(expression_file):
-            raise ScelVisException("cannot find expression file at %s" % expression_file)
-        logger.info("Reading gene expression from %s", expression_file)
-        DGE = pd.read_csv(expression_file, header=0, index_col=0, sep="\t")
+        expression_tsv = os.path.join(self.args.indir, "expression.tsv.gz")
+        expression_mtx = os.path.join(self.args.indir, "expression.mtx")
+        if os.path.isfile(expression_tsv):
+            logger.info("Reading gene expression from %s", expression_tsv)
+            DGE = pd.read_csv(expression_tsv, header=0, index_col=0, sep="\t")
+            X = scipy.sparse.csr_matrix(DGE.values.T)
+            cells = DGE.columns
+            genes = DGE.index
+        elif os.path.isfile(expression_mtx):
+            cells_tsv = os.path.join(self.args.indir, "barcodes.tsv")
+            genes_tsv = os.path.join(self.args.indir, "genes.tsv")
+            if not (os.path.isfile(cells_tsv) and os.path.isfile(genes_tsv)):
+                raise ScelVisException(
+                    "expression.mtx present at %s, but barcodes.tsv or genes.tsv is missing"
+                    % expression_tsv
+                )
+            logger.info("Reading gene expression from %s", expression_mtx)
+            X = scipy.io.mmread(expression_mtx).T.tocsr()
+            cells = pd.read_csv(cells_tsv, header=None).squeeze().values
+            genes = pd.read_csv(genes_tsv, header=None).squeeze().values
+        else:
+            raise ScelVisException(
+                "cannot find expression data at %s or %s" % (expression_tsv, expression_mtx)
+            )
         logger.info("Combining data")
-        ad = sc.AnnData(
-            X=scipy.sparse.csr_matrix(DGE.values.T),
-            obs=pd.concat([coords.loc[DGE.columns], annotation.loc[DGE.columns]], axis=1),
-            var=pd.DataFrame([], index=DGE.index),
-        )
+        ad = sc.AnnData(X=X, obs=annotation.loc[cells], var=pd.DataFrame([], index=genes))
+        coords_types = set([re.sub("[-_][0-9]*$", "", c).upper() for c in coords.columns])
+        for ct in coords_types:
+            ct_cols = [c for c in coords.columns if ct in c.upper()]
+            ad.obsm["X_" + ct] = coords.loc[cells, ct_cols].values
         for col in markers.columns:
             ad.uns["marker_" + col] = markers[col].values
 
         return ad
 
     def _write_output(self, ad):
+        if self.args.ncells:
+            logger.info("Sampling %s cells from anndata object", self.args.ncells)
+            sc.pp.subsample(ad, n_obs=self.args.ncells)
         logger.info("Saving anndata object to %s", self.args.out_file)
         ad.write(self.args.out_file)
 
@@ -373,6 +400,9 @@ class LoomConverter:
         return ad
 
     def _write_output(self, ad):
+        if self.args.ncells:
+            logger.info("Sampling %s cells from anndata object", self.args.ncells)
+            sc.pp.subsample(ad, n_obs=self.args.ncells)
         logger.info("Saving anndata object to %s", self.args.out_file)
         ad.write(self.args.out_file)
 
@@ -472,6 +502,9 @@ def setup_argparse(parser):
         type=int,
         help="Save top n markers per cluster [10]",
     )
+    parser.add_argument(
+        "--ncells", dest="ncells", type=int, help="sample ncells cells from object [take all]"
+    )
     parser.add_argument(
         "--verbose", default=False, action="store_true", help="Enable verbose output"
     )
