@@ -3,8 +3,7 @@
 import urllib.parse
 
 import dash_bootstrap_components as dbc
-import dash_core_components as dcc
-import dash_html_components as html
+from dash import dcc, html
 import plotly.graph_objs as go
 import plotly.subplots as subplots
 
@@ -574,7 +573,7 @@ def render_plot_bars(data, group, split, options, filters_json):
             traces.append(tr)
 
     else:
-        splitvals = ad_here.obs[split].cat.categories
+        splitvals = ad_here.obs[split].unique()
         cm = colors.get_cm(splitvals)
 
         tally = ad_here.obs.groupby([group, split]).size()
