@@ -3,9 +3,7 @@
 import urllib.parse
 
 import dash_bootstrap_components as dbc
-import dash_core_components as dcc
-import dash_html_components as html
-import dash_table
+from dash import dcc, html, dash_table
 import pandas as pd
 import numpy as np
 import plotly.graph_objs as go
@@ -370,10 +368,10 @@ def render_plot_violin_box(data, pathname, plot_type, genelist, group, split, fi
 
     # select color palette
     if split is None:
-        groupvals = ad_here.obs[group].cat.categories
+        groupvals = ad_here.obs[group].unique()
         cm = colors.get_cm(groupvals)
     else:
-        splitvals = ad_here.obs[split].cat.categories
+        splitvals = ad_here.obs[split].unique()
         cm = colors.get_cm(splitvals)
 
     ngenes = len(gl)
@@ -482,7 +480,7 @@ def render_plot_dot(data, pathname, genelist, group, split, filters_json):
 
     ad_here = common.apply_filter_cells_filters(data, filters_json)
 
-    groupvals = ad_here.obs[group].cat.categories
+    groupvals = ad_here.obs[group].unique()
     ngroup = len(groupvals)
     ngenes = len(gl)
 
@@ -563,7 +561,7 @@ def render_plot_dot(data, pathname, genelist, group, split, filters_json):
         )
 
     else:
-        splitvals = ad_here.obs[split].cat.categories
+        splitvals = ad_here.obs[split].unique()
         nsplit = len(splitvals)
         traces = []
         # fix bug in pandas groupby when having more than one grouping variable
